"use strict";
var _ = require("lodash");
var glob = require("glob");
var path = require("path");
var Cucumber = require('cucumber'); // tslint:disable-line:no-var-requires
var CucumberAdapter = (function () {
    function CucumberAdapter(config) {
        var _this = this;
        this.config = config;
        this.argv = [];
        this.run = function (specs) { return new Cucumber.Cli({
            argv: _this.argv.concat(_this.resolved(specs)),
            cwd: _this.config.cwd,
            stdout: process.stdout,
        }).run().then(_this.booleanToPromise); };
        this.notifier = function () { return glob.sync(path.resolve(__dirname) + '/notifier.?s').pop(); };
        this.stageCue = function () { return glob.sync(path.resolve(__dirname) + '/stage_cue.?s').pop(); };
        this.booleanToPromise = function (result) { return result ? Promise.resolve(result) : Promise.reject(result); };
        this.argv = ['node', 'cucumberjs']
            .concat(['--require', this.notifier()])
            .concat(['--require', this.stageCue()])
            .concat(this.argumentsFrom(config));
    }
    CucumberAdapter.prototype.argumentsFrom = function (config) {
        var resolvedConfig = Object.assign({}, config.options, { require: this.resolved(config.options.require || []) });
        var toCLIArgument = function (option) { return function (value) { return !!value
            ? ["--" + option, value]
            : ["--no-" + option]; }; };
        return _.chain(resolvedConfig).toPairs().
            flatMap(function (option) { return _.castArray(option[1]).map(toCLIArgument(option[0])); }).
            flatten().
            value();
    };
    CucumberAdapter.prototype.resolved = function (paths) {
        var _this = this;
        var resolveGlobs = function (path) { return glob.sync(path, { cwd: _this.config.cwd }); };
        return _.chain(paths).map(resolveGlobs).flatten().value();
    };
    return CucumberAdapter;
}());
exports.CucumberAdapter = CucumberAdapter;
//# sourceMappingURL=cucumber_adapter.js.map