"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var cucumber_1 = require("cucumber");
var serenity_js_1 = require("serenity-js");
var domain_1 = require("serenity-js/lib/serenity/domain");
var CucumberStep = require('cucumber/lib/models/step').default; // tslint:disable-line:no-var-requires
cucumber_1.defineSupportCode(function (_a) {
    var registerHandler = _a.registerHandler;
    function isOfInterest(step) {
        // "Before hooks" will also trigger the listeners, but we don't want to report on them
        // https://github.com/cucumber/cucumber-js/blob/master/docs/support_files/event_handlers.md
        return step instanceof CucumberStep && !step.isHidden;
    }
    registerHandler('BeforeScenario', function (scenario) { return serenity_js_1.serenity.notify(new domain_1.SceneStarts(CucumberScene.from(scenario))); });
    registerHandler('BeforeStep', function (step) {
        return isOfInterest(step) && serenity_js_1.serenity.notify(new domain_1.ActivityStarts(CucumberTask.from(step)));
    });
    registerHandler('StepResult', function (result) {
        return isOfInterest(result.step) && serenity_js_1.serenity.notify(new domain_1.ActivityFinished(CucumberOutcome.fromStep(result)));
    });
    registerHandler('ScenarioResult', function (result) {
        return serenity_js_1.serenity.notify(new domain_1.SceneFinished(CucumberOutcome.fromScenario(result)));
    });
});
var CucumberOutcome = (function (_super) {
    __extends(CucumberOutcome, _super);
    function CucumberOutcome(subject, result, error) {
        return _super.call(this, subject, result, error) || this;
    }
    return CucumberOutcome;
}(domain_1.Outcome));
CucumberOutcome.fromStep = function (result) { return new CucumberOutcome(CucumberTask.from(result.step), CucumberResult.from(result), realErrorFrom(result.failureException)); };
CucumberOutcome.fromScenario = function (result) { return new CucumberOutcome(CucumberScene.from(result.scenario), CucumberResult.from(result), realErrorFrom(result.failureException)); };
function realErrorFrom(pseudoError) {
    switch (typeof pseudoError) {
        case 'string': return new Error(pseudoError);
        case 'object': return pseudoError;
        default: return undefined;
    }
}
var CucumberResult = (function () {
    function CucumberResult() {
    }
    return CucumberResult;
}());
CucumberResult.from = function (result) {
    var isTimeOut = function (e) { return e && /timed out/.test(e.message); };
    var results = {
        undefined: domain_1.Result.PENDING,
        failed: domain_1.Result.FAILURE,
        pending: domain_1.Result.PENDING,
        passed: domain_1.Result.SUCCESS,
        skipped: domain_1.Result.SKIPPED,
    };
    if (!results[result.status]) {
        throw new Error("Couldn't map the '" + result.status + "' to a Serenity Result");
    }
    return isTimeOut(realErrorFrom(result.failureException))
        ? domain_1.Result.ERROR
        : results[result.status];
};
var CucumberScene = (function (_super) {
    __extends(CucumberScene, _super);
    function CucumberScene(scenario) {
        return _super.call(this, scenario.name, scenario.feature.name, scenario.uri, scenario.tags.map(function (tag) { return domain_1.Tag.from(tag.name); }), scenario.feature.name + ":" + scenario.line + ":" + scenario.name) || this;
    }
    return CucumberScene;
}(domain_1.RecordedScene));
CucumberScene.from = function (scenario) { return new CucumberScene(scenario); };
var CucumberTask = (function (_super) {
    __extends(CucumberTask, _super);
    function CucumberTask(step) {
        return _super.call(this, CucumberTask.fullNameOf(step)) || this;
    }
    return CucumberTask;
}(domain_1.RecordedTask));
CucumberTask.from = function (step) { return new CucumberTask(step); };
CucumberTask.fullNameOf = function (step) { return [
    step.keyword,
    step.name,
    step.arguments.map(function (argument) { return SerialisedStepArgument.from(argument); }).join('\n'),
].join('').trim(); };
var SerialisedStepArgument = (function () {
    function SerialisedStepArgument() {
    }
    return SerialisedStepArgument;
}());
SerialisedStepArgument.from = function (argument) {
    switch (argument.constructor.name) {
        case 'DataTable': return "\n" + argument.raw().map(function (row) { return "| " + row.join(' | ') + " |"; }).join('\n');
        case 'DocString': return "\n" + argument.content;
    }
};
//# sourceMappingURL=notifier.js.map